@@ -26,10 +26,11 @@ from __future__ import (
 from astropy.coordinates import SkyCoord
 from astropy.units import Unit, Quantity
 from .query import DALResults, DALQuery, DALService, Record
-from .datalink import DatalinkMixin
+from .adhoc import DatalinkResultsMixin, DatalinkRecordMixin
 
 __all__ = ["search", "SCSService", "SCSQuery", "SCSResults", "SCSRecord"]
 
+
 def search(url, pos, radius=1.0, verbosity=2, **keywords):
     """
     submit a simple Cone Search query that requests objects or observations
@@ -243,19 +244,26 @@ class SCSQuery(DALQuery):
         `~astropy.coordinates.SkyCoord` instance.
         """
         return getattr(self, "_pos", None)
+
     @pos.setter
-    def pos(self, val):
-        setattr(self, "_pos", val)
+    def pos(self, pos):
+        setattr(self, "_pos", pos)
 
-        # use the astropy, luke
-        if not isinstance(val, SkyCoord):
-            pos_ra, pos_dec = val
+        if not isinstance(pos, SkyCoord):
+            try:
+                ra, dec = pos
+            except (TypeError, ValueError):
+                raise ValueError(
+                    'Pos must be a sequence with exactly two values, '
+                    'expressing ra and dec in icrs degrees'
+                )
 
-            # assume ICRS degrees
-            val = SkyCoord(ra=pos_ra, dec=pos_dec, unit="deg", frame="icrs")
+            # assume degrees
+            pos = SkyCoord(ra=ra, dec=dec, unit="deg", frame="icrs")
+
+        self["RA"] = pos.icrs.ra.deg
+        self["DEC"] = pos.icrs.dec.deg
 
-        self["RA"] = val.icrs.ra.deg
-        self["DEC"] = val.icrs.dec.deg
     @pos.deleter
     def pos(self):
         delattr(self, "_pos")
@@ -269,22 +277,28 @@ class SCSQuery(DALQuery):
         `~astropy.units.Quantity` instance.
         """
         return getattr(self, "_radius", None)
+
     @radius.setter
-    def radius(self, val):
-        setattr(self, "_radius", val)
+    def radius(self, radius):
+        setattr(self, "_radius", radius)
+
+        if not isinstance(radius, Quantity):
+            valerr = ValueError("Radius must be exactly one value")
 
-        if not isinstance(val, Quantity):
-            # assume degrees
-            val = val * Unit("deg")
             try:
-                if len(val):
-                    raise ValueError(
-                        "radius may be specified using exactly one value")
+                # assume degrees
+                radius = radius * Unit("deg")
+            except ValueError:
+                raise valerr
+
+            try:
+                if len(radius):
+                    raise valerr
             except TypeError:
-                # len 1
-                pass
+                pass  # len 1
+
+        self["SR"] = radius.to(Unit("deg")).value
 
-        self["SR"] = val.to(Unit("deg")).value
     @radius.deleter
     def radius(self):
         delattr(self, "_radius")
@@ -298,10 +312,12 @@ class SCSQuery(DALQuery):
         set of columsn, 3 means as many columns as are  available.
         """
         return getattr(self, "_verbosity", None)
+
     @verbosity.setter
-    def verbosity(self, val):
-        setattr(self, "_verbosity", val)
-        self["VERB"] = val
+    def verbosity(self, verbosity):
+        setattr(self, "_verbosity", verbosity)
+        self["VERB"] = verbosity
+
     @verbosity.deleter
     def verbosity(self):
         delattr(self, "_verbosity")
@@ -324,7 +340,7 @@ class SCSQuery(DALQuery):
         return SCSResults(self.execute_votable(), url=self.queryurl)
 
 
-class SCSResults(DALResults, DatalinkMixin):
+class SCSResults(DALResults, DatalinkResultsMixin):
     """
     The list of matching catalog records resulting from a catalog (SCS) query.
     Each record contains a set of metadata that describes a source or
@@ -370,12 +386,6 @@ class SCSResults(DALResults, DatalinkMixin):
     and the data from the column matching that name is returned as
     a Numpy array.
     """
-    def __init__(self, votable, **kwargs):
-        """
-        Initialize datalinks
-        """
-        super(SCSResults, self).__init__(votable, **kwargs)
-        self._init_datalinks(votable)
 
     def _findresultsresource(self, votable):
         if len(votable.resources) < 1:
@@ -390,7 +400,6 @@ class SCSResults(DALResults, DatalinkMixin):
         if info:
             return (info.name, info.value)
 
-
         # look next in the result resource
         res = self._findresultsresource(votable)
         if res:
@@ -448,7 +457,7 @@ class SCSResults(DALResults, DatalinkMixin):
         return SCSRecord(self, index)
 
 
-class SCSRecord(Record):
+class SCSRecord(DatalinkRecordMixin, Record):
     """
     a dictionary-like container for data in a record from the results of an
     Cone Search (SCS) query, describing a matching source or observation.
