@@ -32,15 +32,18 @@ from __future__ import (
     absolute_import, division, print_function, unicode_literals)
 
 import re
+
 from astropy.extern import six
 from astropy.coordinates import SkyCoord
 from astropy.time import Time
 from astropy.units import Quantity, Unit
+
 from .query import DALResults, DALQuery, DALService, Record, mime2extension
-from .datalink import DatalinkMixin
+from .adhoc import DatalinkResultsMixin, DatalinkRecordMixin, SodaRecordMixin
 
 __all__ = ["search", "SIAService", "SIAQuery", "SIAResults", "SIARecord"]
 
+
 def search(
         url, pos, size=1.0, format='all', intersect="overlaps", verbosity=2,
         **keywords):
@@ -54,14 +57,11 @@ def search(
     pos : `~astropy.coordinates.SkyCoord` class or sequence of two floats
         the position of the center of the rectangular search region.
         assuming icrs decimal degrees if unit is not specified.
-    size : `~astropy.units.Quantity` class or 2 element sequence of float
+    size : `~astropy.units.Quantity` class or up to 2 floats.
         the full rectangular size of the search region along the
         RA and Dec directions.
         converted if it's a iterable containing scalars,
         assuming decimal degrees.
-    size : `~astropy.units.Quantity` class or scalar float
-        the size of the rectangular region around pos.
-        assuming icrs decimal degrees if unit is not specified.
     format : str
         the image format(s) of interest.  "all" (default)
         indicates all available formats; "graphic" indicates
@@ -113,6 +113,7 @@ def search(
     service = SIAService(url)
     return service.search(pos, size, format, intersect, verbosity, **keywords)
 
+
 class SIAService(DALService):
     """
     a representation of an SIA service
@@ -171,18 +172,14 @@ class SIAService(DALService):
             self, pos, size=1.0, format='all', intersect="overlaps",
             verbosity=2, **keywords):
         """
-        submit a simple SIA query to this service with the given constraints.
-
-        This method is provided for a simple but typical SIA queries.  For
-        more complex queries, one should create an SIAQuery object via
-        create_query()
+        submit a SIA query to this service with the given parameters.
 
         Parameters
         ----------
         pos : `~astropy.coordinates.SkyCoord` class or sequence of two floats
             the position of the center of the rectangular search region.
             assuming icrs decimal degrees if unit is not specified.
-        size : `~astropy.units.Quantity` class or 2 element sequence of float
+        size : `~astropy.units.Quantity` class or up to 2 floats.
             the full rectangular size of the search region along the
             RA and Dec directions.
             converted if it's a iterable containing scalars,
@@ -212,7 +209,7 @@ class SIAService(DALService):
         verbosity : int
             an integer value that indicates the volume of columns
             to return in the result table.  0 means the minimum
-            set of columsn, 3 means as many columns as are  available.
+            set of columns, 3 means as many columns as are  available.
         **keywords :
             additional parameters can be given via arbitrary
             case insensitive keyword arguments. Where there is overlap
@@ -243,8 +240,7 @@ class SIAService(DALService):
 
     def create_query(
             self, pos=None, size=None, format=None, intersect=None,
-            verbosity=None, **keywords
-        ):
+            verbosity=None, **keywords):
         """
         create a query object that constraints can be added to and then
         executed.  The input arguments will initialize the query with the
@@ -255,7 +251,7 @@ class SIAService(DALService):
         pos : `~astropy.coordinates.SkyCoord` class or sequence of two floats
             the position of the center of the rectangular search region.
             assuming icrs decimal degrees if unit is not specified.
-        size : `~astropy.units.Quantity` class or 2 element sequence of float
+        size : `~astropy.units.Quantity` class or up to 2 floats.
             the full rectangular size of the search region along the
             RA and Dec directions.
             converted if it's a iterable containing scalars,
@@ -326,9 +322,8 @@ class SIAQuery(DALQuery):
     """
 
     def __init__(
-            self, baseurl, pos=None, size=None, format=None,
-            intersect=None, verbosity=None, **keywords
-        ):
+            self, baseurl, pos=None, size=None, format=None, intersect=None,
+            verbosity=None, **keywords):
         """
         initialize the query object with a baseurl and the given parameters
 
@@ -339,7 +334,7 @@ class SIAQuery(DALQuery):
         pos : `~astropy.coordinates.SkyCoord` class or sequence of two floats
             the position of the center of the rectangular search region.
             assuming icrs decimal degrees if unit is not specified.
-        size : `~astropy.units.Quantity` class or 2 element sequence of float
+        size : `~astropy.units.Quantity` class or up to 2 floats.
             the full rectangular size of the search region along the
             RA and Dec directions.
             converted if it's a iterable containing scalars,
@@ -400,23 +395,26 @@ class SIAQuery(DALQuery):
         `~astropy.coordinates.SkyCoord` instance.
         """
         return getattr(self, "_pos", None)
+
     @pos.setter
-    def pos(self, val):
-        setattr(self, "_pos", val)
+    def pos(self, pos):
+        setattr(self, "_pos", pos)
 
-        # use the astropy, luke
-        if not isinstance(val, SkyCoord):
+        if not isinstance(pos, SkyCoord):
             try:
-                pos_ra, pos_dec = val
-            except TypeError:
+                ra, dec = pos
+            except (TypeError, ValueError):
                 raise ValueError(
-                    "pos may be specified using exactly two values")
+                    'Pos must be a sequence with exactly two values, '
+                    'expressing ra and dec in icrs degrees'
+                )
 
-            # assume ICRS degrees
-            val = SkyCoord(ra=pos_ra, dec=pos_dec, unit="deg", frame="icrs")
+            # assume degrees
+            pos = SkyCoord(ra=ra, dec=dec, unit="deg", frame="icrs")
 
         self["POS"] = "{ra},{dec}".format(
-            ra=val.icrs.ra.deg, dec=val.icrs.dec.deg)
+            ra=pos.icrs.ra.deg, dec=pos.icrs.dec.deg)
+
     @pos.deleter
     def pos(self):
         delattr(self, "_pos")
@@ -429,26 +427,35 @@ class SIAQuery(DALQuery):
         `~astropy.units.Quantity` instance.
         """
         return getattr(self, "_size", None)
-    @size.setter
-    def size(self, val):
-        setattr(self, "_size", val)
 
-        if not isinstance(val, Quantity):
-            # assume degrees
-            val = val * Unit("deg")
+    @size.setter
+    def size(self, size):
+        setattr(self, "_size", size)
+
+        if not isinstance(size, Quantity):
+            valerr = ValueError(
+                'Size must be either a single value or a sequence with two'
+                'values, expressing degrees'
+            )
+
             try:
-                if len(val) > 2:
-                    raise ValueError(
-                        "size may be specified using either one or two values")
+                # assume degrees
+                size = size * Unit("deg")
+            except ValueError:
+                raise valerr
+
+            try:
+                if len(size) > 2:
+                    raise valerr
             except TypeError:
-                # len 1
-                pass
+                pass  # len 1
 
         try:
             self["SIZE"] = ",".join(
-                str(deg) for deg in val.to(Unit("deg")).value)
+                str(deg) for deg in size.to(Unit("deg")).value)
         except TypeError:
-            self["SIZE"] = str(val.to(Unit("deg")).value)
+            self["SIZE"] = str(size.to(Unit("deg")).value)
+
     @size.deleter
     def size(self):
         delattr(self, "_size")
@@ -466,14 +473,16 @@ class SIAQuery(DALQuery):
         * can have values like "fits", "jpeg", "png", etc.
         """
         return getattr(self, "_format", None)
+
     @format.setter
-    def format(self, val):
-        setattr(self, "_format", val)
+    def format(self, format_):
+        setattr(self, "_format", format_)
 
-        if type(val) in (six.text_type, six.binary_type):
-            val = [val]
+        if type(format_) in (six.text_type, six.binary_type):
+            format_ = [format_]
+
+        self["FORMAT"] = ",".join(_.upper() for _ in format_)
 
-        self["FORMAT"] = ",".join(_.upper() for _ in val)
     @format.deleter
     def format(self):
         delattr(self, "_format")
@@ -493,10 +502,12 @@ class SIAQuery(DALQuery):
         ========= ======================================================
         """
         return getattr(self, "_intersect", None)
+
     @intersect.setter
-    def intersect(self, val):
-        setattr(self, "_intersect", val)
-        self["INTERSECT"] = val.upper()
+    def intersect(self, intersect):
+        setattr(self, "_intersect", intersect)
+        self["INTERSECT"] = intersect.upper()
+
     @intersect.deleter
     def intersect(self):
         delattr(self, "_intersect")
@@ -510,10 +521,12 @@ class SIAQuery(DALQuery):
         set of columsn, 3 means as many columns as are  available.
         """
         return getattr(self, "_verbosity", None)
+
     @verbosity.setter
-    def verbosity(self, val):
-        setattr(self, "_verbosity", val)
-        self["VERB"] = val
+    def verbosity(self, verbosity):
+        setattr(self, "_verbosity", verbosity)
+        self["VERB"] = verbosity
+
     @verbosity.deleter
     def verbosity(self):
         delattr(self, "_verbosity")
@@ -536,7 +549,7 @@ class SIAQuery(DALQuery):
         return SIAResults(self.execute_votable(), url=self.queryurl)
 
 
-class SIAResults(DALResults, DatalinkMixin):
+class SIAResults(DatalinkResultsMixin, DALResults):
     """
     The list of matching images resulting from an image (SIA) query.
     Each record contains a set of metadata that describes an available
@@ -583,19 +596,12 @@ class SIAResults(DALResults, DatalinkMixin):
     a Numpy array.
     """
 
-    def __init__(self, votable, **kwargs):
-        """
-        Initialize datalinks
-        """
-        super(SIAResults, self).__init__(votable, **kwargs)
-        self._init_datalinks(votable)
-
     def getrecord(self, index):
         """
         return a representation of a sia result record that follows
         dictionary semantics. The keys of the dictionary are those returned by
-        this instance's fieldnames attribute. The returned record has additional
-        image-specific properties
+        this instance's fieldnames attribute. The returned record has
+        additional image-specific properties
 
         Parameters
         ----------
@@ -620,7 +626,8 @@ class SIAResults(DALResults, DatalinkMixin):
         """
         return SIARecord(self, index)
 
-class SIARecord(Record):
+
+class SIARecord(SodaRecordMixin, DatalinkRecordMixin, Record):
     """
     a dictionary-like container for data in a record from the results of an
     image (SIA) search, describing an available image.
@@ -633,6 +640,12 @@ class SIARecord(Record):
     operator) where *key* is table column name.
     """
 
+    def getdataformat(self):
+        """
+        return the mimetype of the dataset described by this record.
+        """
+        return self.getbyucd("VOX:Image_Format", decode=True)
+
     @property
     def pos(self):
         """
@@ -780,7 +793,7 @@ class SIARecord(Record):
     @property
     def bandpass_hilimit(self):
         """
-        the upper limit of the bandpass, as an astropy Quantity in bandpass_unit
+        the upper limit of the bandpass as astropy Quantity in bandpass_unit
         """
         return Quantity(
             self.getbyucd("VOX:BandPass_HiLimit"), self.bandpass_unit)
@@ -788,7 +801,7 @@ class SIARecord(Record):
     @property
     def bandpass_lolimit(self):
         """
-        the lower limit of the bandpass, as an astropy Quantity in bandpass_unit
+        the lower limit of the bandpass as astropy Quantity in bandpass_unit
         """
         return Quantity(
             self.getbyucd("VOX:BandPass_LoLimit"), self.bandpass_unit)
@@ -810,7 +823,7 @@ class SIARecord(Record):
         * X -- The image pixels were computed by the service directly from a
                primary data set hence were not filtered by an interpolator.
         * Z -- The image pixels contain valid flux (intensity) values, e.g., if
-               the pixels were resampled a flux-preserving interpolator was used.
+               the pixels were resampled with a flux-preserving interpolator.
         * V -- The image pixels contain some unspecified visualization of the
                data, hence are suitable for display but not for numerical
                analysis.
@@ -845,7 +858,11 @@ class SIARecord(Record):
         to retrieve the dataset described by this record.  None is returned
         if no such column exists.
         """
-        return self.acref
+        dataurl = super(SIARecord, self).getdataurl()
+        if dataurl is None:
+            return self.acref
+        else:
+            return dataurl
 
     def suggest_dataset_basename(self):
         """
