@@ -9,25 +9,31 @@ from functools import partial
 from datetime import datetime
 from time import sleep
 from distutils.version import LooseVersion
+
 import requests
+
 from astropy.io.votable import parse as votableparse
 
 from .query import (
-    DALResults, DALQuery, DALService, Record, UploadList,
-    DALServiceError, DALQueryError)
-from .mixin import AvailabilityMixin, CapabilityMixin
-from .datalink import DatalinkMixin
-from ..tools import vosi, uws
+    DALResults, DALQuery, DALService, Record, UploadList, DALServiceError,
+    DALQueryError)
+from .vosi import AvailabilityMixin, CapabilityMixin, VOSITables
+from .adhoc import DatalinkResultsMixin, DatalinkRecordMixin, SodaRecordMixin
+
+from ..io import vosi, uws
+from ..io.vosi import tapregext as tr
 
 __all__ = [
     "search", "escape", "TAPService", "TAPQuery", "AsyncTAPJob", "TAPResults"]
 
+
 def escape(term):
     """
     escapes a term for use in ADQL
     """
     return str(term).replace("'", "''")
 
+
 def search(url, query, language="ADQL", maxrec=None, uploads=None, **keywords):
     """
     submit a Table Access query that returns rows matching the criteria given.
@@ -42,7 +48,7 @@ def search(url, query, language="ADQL", maxrec=None, uploads=None, **keywords):
         specifies the query language, default ADQL.
         useful for services which allow to use the backend query language.
     maxrec : int
-        specifies the maximum records to return. defaults to the service default
+        the maximum records to return. defaults to the service default
     uploads : dict
         a mapping from table names to file like objects containing a votable
 
@@ -63,6 +69,7 @@ def search(url, query, language="ADQL", maxrec=None, uploads=None, **keywords):
     service = TAPService(url)
     return service.search(query, language, maxrec, uploads, **keywords)
 
+
 class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
     """
     a representation of a Table Access Protocol service
@@ -84,7 +91,7 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
     @property
     def tables(self):
         """
-        returns tables as a flat OrderedDict
+        returns tables as a dict-like object
         """
         if self._tables is None:
             tables_url = '{0}/tables'.format(self.baseurl)
@@ -99,7 +106,8 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
             # requests doesn't decode the content by default
             response.raw.read = partial(response.raw.read, decode_content=True)
 
-            self._tables = vosi.parse_tables(response.raw)
+            self._tables = VOSITables(
+                vosi.parse_tables(response.raw.read), tables_url)
         return self._tables
 
     @property
@@ -114,9 +122,9 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
         """
         try:
             for capa in self.capabilities:
-                if "outputLimit" in capa:
-                    return capa["outputLimit"]["default"]["value"]
-        except KeyError:
+                if isinstance(capa, tr.TableAccess):
+                    return capa.outputlimit.default.content
+        except AttributeError:
             pass
         raise DALServiceError("Default limit not exposed by the service")
 
@@ -132,9 +140,9 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
         """
         try:
             for capa in self.capabilities:
-                if "outputLimit" in capa:
-                    return capa["outputLimit"]["hard"]["value"]
-        except KeyError:
+                if isinstance(capa, tr.TableAccess):
+                    return capa.outputlimit.hard.content
+        except AttributeError:
             pass
         raise DALServiceError("Hard limit not exposed by the service")
 
@@ -143,11 +151,11 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
         """
         a list of upload methods in form of IVOA identifiers
         """
-        _upload_methods = []
+        upload_methods = []
         for capa in self.capabilities:
-            if "uploadMethods" in capa:
-                _upload_methods += capa["uploadMethods"]
-        return _upload_methods
+            if isinstance(capa, tr.TableAccess):
+                upload_methods += capa.uploadmethods
+        return upload_methods
 
     def run_sync(
             self, query, language="ADQL", maxrec=None, uploads=None,
@@ -163,9 +171,9 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
             specifies the query language, default ADQL.
             useful for services which allow to use the backend query language.
         maxrec : int
-            specifies the maximum records to return. defaults to the service default
+            the maximum records to return. defaults to the service default
         uploads : dict
-            a mapping from table names to file like objects containing a votable
+            a mapping from table names to objects containing a votable
 
         Returns
         -------
@@ -180,7 +188,7 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
             query, language=language, maxrec=maxrec, uploads=uploads,
             **keywords).execute()
 
-    #alias for service discovery
+    # alias for service discovery
     search = run_sync
 
     def run_async(
@@ -197,9 +205,9 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
             specifies the query language, default ADQL.
             useful for services which allow to use the backend query language.
         maxrec : int
-            specifies the maximum records to return. defaults to the service default
+            the maximum records to return. defaults to the service default
         uploads : dict
-            a mapping from table names to file like objects containing a votable
+            a mapping from table names to objects containing a votable
 
         Returns
         -------
@@ -229,7 +237,9 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
 
         return result
 
-    def submit_job(self, query, language="ADQL", maxrec=None, uploads=None):
+    def submit_job(
+            self, query, language="ADQL", maxrec=None, uploads=None,
+            **keywords):
         """
         submit a async query without starting it and returns a AsyncTAPJob
         object
@@ -242,9 +252,9 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
             specifies the query language, default ADQL.
             useful for services which allow to use the backend query language.
         maxrec : int
-            specifies the maximum records to return. defaults to the service default
+            the maximum records to return. defaults to the service default
         uploads : dict
-            a mapping from table names to file like objects containing a votable
+            a mapping from table names to objects containing a votable
 
         Returns
         -------
@@ -256,7 +266,7 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
         AsyncTAPJob
         """
         return AsyncTAPJob.create(
-            self.baseurl, query, language, maxrec, uploads)
+            self.baseurl, query, language, maxrec, uploads, **keywords)
 
     def create_query(
             self, query=None, mode="sync", language="ADQL", maxrec=None,
@@ -311,9 +321,9 @@ class AsyncTAPJob(object):
             specifies the query language, default ADQL.
             useful for services which allow to use the backend query language.
         maxrec : int
-            specifies the maximum records to return. defaults to the service default
+            the maximum records to return. defaults to the service default
         uploads : dict
-            a mapping from table names to file like objects containing a votable
+            a mapping from table names to objects containing a votable
         """
         query = TAPQuery(
             baseurl, query, mode="async", language=language, maxrec=maxrec,
@@ -324,7 +334,7 @@ class AsyncTAPJob(object):
 
     def __init__(self, url):
         """
-        initialize the job object with the given url and fetch the remote values
+        initialize the job object with the given url and fetch remote values
 
         Parameters
         ----------
@@ -367,14 +377,14 @@ class AsyncTAPJob(object):
         # requests doesn't decode the content by default
         response.raw.read = partial(response.raw.read, decode_content=True)
 
-        self._job.update(uws.parse_job(response.raw))
+        self._job = uws.parse_job(response.raw.read)
 
     @property
     def job(self):
         """
         all up-to-date uws job infos as dictionary
         """
-        #keep it up to date
+        # keep it up to date
         self._update()
         return self._job
 
@@ -390,7 +400,7 @@ class AsyncTAPJob(object):
         """
         the job id
         """
-        return self._job["jobId"]
+        return self._job.jobid
 
     @property
     def phase(self):
@@ -398,26 +408,18 @@ class AsyncTAPJob(object):
         the current query phase
         """
         self._update()
-        return self._job["phase"]
+        return self._job.phase
 
     @property
     def execution_duration(self):
         """
-        maximum execution duration. read-write
+        maximum execution duration as ~`astropy.time.TimeDelta`
         """
         self._update()
-        return self._job["executionDuration"]
+        return self._job.executionduration
 
     @execution_duration.setter
     def execution_duration(self, value):
-        """
-        maximum execution duration. read-write
-
-        Parameters
-        ----------
-        value : int
-            seconds after the query execution is aborted
-        """
         try:
             response = requests.post(
                 "{}/executionduration".format(self.url),
@@ -425,7 +427,8 @@ class AsyncTAPJob(object):
             response.raise_for_status()
         except requests.exceptions.RequestException as ex:
             raise DALServiceError.from_except(ex, self.url)
-        self._job["executionDuration"] = value
+
+        self._update()
 
     @property
     def destruction(self):
@@ -434,7 +437,7 @@ class AsyncTAPJob(object):
         read-write
         """
         self._update()
-        return self._job["destruction"]
+        return self._job.destruction
 
     @destruction.setter
     def destruction(self, value):
@@ -448,7 +451,7 @@ class AsyncTAPJob(object):
             datetime after which the job results are deleted automatically
         """
         try:
-            #is string? easier to ask for forgiveness
+            # is string? easier to ask for forgiveness
             value = datetime.strptime(value, "%Y-%m-%dT%H:%M:%SZ")
         except ValueError:
             pass
@@ -460,7 +463,8 @@ class AsyncTAPJob(object):
             response.raise_for_status()
         except requests.exceptions.RequestException as ex:
             raise DALServiceError.from_except(ex, self.url)
-        self._job["destruction"] = value
+
+        self._update()
 
     @property
     def quote(self):
@@ -468,7 +472,7 @@ class AsyncTAPJob(object):
         estimated runtime
         """
         self._update()
-        return self._job["quote"]
+        return self._job.quote
 
     @property
     def owner(self):
@@ -476,14 +480,31 @@ class AsyncTAPJob(object):
         job owner (if applicable)
         """
         self._update()
-        return self._job["owner"]
+        return self._job.owner
+
+    @property
+    def results(self):
+        """
+        The job results if exists
+        """
+        return self._job.results
+
+    @property
+    def result(self):
+        """
+        The job result if exists
+        """
+        try:
+            return self._job.results[0]
+        except IndexError:
+            return None
 
     @property
     def result_uris(self):
         """
         a list of the last result uri's
         """
-        return self._job["results"]
+        return [result.href for result in self._job.results]
 
     @property
     def result_uri(self):
@@ -491,14 +512,14 @@ class AsyncTAPJob(object):
         the first result uri
         """
         try:
-            return next(iter(self.result_uris.values()))
-        except StopIteration:
+            return self.result_uris[0]
+        except IndexError:
             return None
 
     @property
     def uws_version(self):
         self._update()
-        return self._job["version"]
+        return self._job.version
 
     def run(self):
         """
@@ -553,7 +574,7 @@ class AsyncTAPJob(object):
         while True:
             self._update(wait_for_statechange=True)
             # use the cached value
-            cur_phase = self._job["phase"]
+            cur_phase = self._job.phase
 
             if cur_phase not in active_phases:
                 raise DALServiceError(
@@ -563,7 +584,7 @@ class AsyncTAPJob(object):
                 break
 
             # fallback for uws 1.0
-            if LooseVersion(self._job["version"]) < LooseVersion("1.1"):
+            if LooseVersion(self._job.version) < LooseVersion("1.1"):
                 sleep(interval)
                 interval = min(120, interval * increment)
 
@@ -590,10 +611,8 @@ class AsyncTAPJob(object):
         DALQueryError
             if theres an error
         """
-        if self.phase in ["ERROR", "ABORTED"]:
-            raise DALQueryError(
-                self._job.get("message", "Unknown Query Error"),
-                self.phase, self.url)
+        if self.phase in {"ERROR", "ABORTED"}:
+            raise DALQueryError("Query Error", self.phase, self.url)
 
     def fetch_result(self):
         """
@@ -733,7 +752,7 @@ class TAPQuery(DALQuery):
         return response
 
 
-class TAPResults(DALResults, DatalinkMixin):
+class TAPResults(DatalinkResultsMixin, DALResults):
     """
     The list of matching images resulting from an image (SIA) query.
     Each record contains a set of metadata that describes an available
@@ -776,13 +795,6 @@ class TAPResults(DALResults, DatalinkMixin):
     a Numpy array.
     """
 
-    def __init__(self, votable, **kwargs):
-        """
-        Initialize datalinks
-        """
-        super(TAPResults, self).__init__(votable, **kwargs)
-        self._init_datalinks(votable)
-
     @property
     def infos(self):
         """
@@ -801,8 +813,8 @@ class TAPResults(DALResults, DatalinkMixin):
         """
         return a representation of a tap result record that follows
         dictionary semantics. The keys of the dictionary are those returned by
-        this instance's fieldnames attribute. The returned record has additional
-        image-specific properties
+        this instance's fieldnames attribute. The returned record has
+        additional image-specific properties
 
         Parameters
         ----------
@@ -825,4 +837,8 @@ class TAPResults(DALResults, DatalinkMixin):
         --------
         Record
         """
-        return Record(self, index)
+        return TAPRecord(self, index)
+
+
+class TAPRecord(SodaRecordMixin, DatalinkRecordMixin, Record):
+    pass
